@@ -2,12 +2,15 @@ import abc
 import atexit
 import contextlib
 import contextvars
+import csv
+import enum
 import functools
 import inspect
 import os
 import math
 import platform
 import signal
+import subprocess
 import textwrap
 import threading
 import typing
@@ -47,6 +50,16 @@ evaluation_context = contextvars.ContextVar('evaluation_context',
                                             default=globalenv)
 
 
+class _ENVVAR_ACTION(enum.Enum):
+    KEEP_WARN = 0
+    REPLACE_WARN = 1
+    KEEP_NOWARN = 2
+    REPLACE_NOWARN = 3
+
+
+_DEFAULT_ENVVAR_ACTION: _ENVVAR_ACTION = _ENVVAR_ACTION.REPLACE_WARN
+
+
 def get_evaluation_context() -> SexpEnvironment:
     """Get the frame (environment) in which R code is currently evaluated."""
     warnings.warn('This function is deprecated. '
@@ -1066,6 +1079,7 @@ def initr_checkenv(
 
     status = None
     with openrlib.rlock:
+        _setrenvvars(_DEFAULT_ENVVAR_ACTION)
         if embedded.is_r_externally_initialized():
             embedded._setinitialized()
         else:
@@ -1100,6 +1114,69 @@ def _update_R_ENC_PY():
     conversion._R_ENC_PY[openrlib.rlib.CE_ANY] = 'utf-8'
 
 
+# TODO: This function could be used by situation.py. May be better to
+# place elsewhere.
+def _getrenvvars(
+        baselinevars: typing.Optional[typing.MutableMapping[str, str]] = None,
+        r_home: typing.Optional[str] = None
+) -> typing.Tuple[typing.Tuple[str, str], ...]:
+    """Get the environment variables defined by the R front-end script."""
+
+    if baselinevars is None:
+        baselinevars = os.environ
+    if r_home is None:
+        r_home = openrlib.R_HOME
+        if r_home is None:
+            raise RuntimeError('Unable to determine R_HOME.')
+    cmd = (
+        os.path.join(r_home, 'bin', 'Rscript'),
+        '-e',
+        'x<-Sys.getenv();y<-as.character(x);names(y)<-names(x);'
+        'write.csv(y)'
+    )
+
+    envvars = subprocess.check_output(cmd,
+                                      universal_newlines=True,
+                                      stderr=subprocess.PIPE)
+
+    res = []
+    reader = csv.reader(row for row in envvars.split(os.linesep) if row != '')
+    # Skip column names.
+    next(reader)
+    for k, v in reader:
+        if (
+                (k not in baselinevars)
+                or
+                (baselinevars[k] != v)
+        ):
+            res.append((k, v))
+    return tuple(res)
+
+
+def _setrenvvars(action: _ENVVAR_ACTION):
+    new_envvars = {}
+    for k, v in _getrenvvars():
+        if k in os.environ:
+            if (
+                    action in (_ENVVAR_ACTION.KEEP_WARN, _ENVVAR_ACTION.KEEP_NOWARN)
+            ):
+                if action is _ENVVAR_ACTION.KEEP_WARN:
+                    warnings.warn(
+                        f'Environment variable "{k}" redefined by R but ignored.'
+                    )
+                continue
+            elif (
+                    action in (_ENVVAR_ACTION.REPLACE_WARN, _ENVVAR_ACTION.REPLACE_NOWARN)
+            ):
+                if action is _ENVVAR_ACTION.REPLACE_WARN:
+                    warnings.warn(
+                        f'Environment variable "{k}" redefined by R and overriding '
+                        'existing variable.'
+                    )
+        new_envvars[k] = v
+    os.environ.update(new_envvars)
+
+
 def _post_initr_setup() -> None:
 
     emptyenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_EmptyEnv)
